<?php

/**
 * Сущность настроек приватности пользователя
 *
 * @package modules.user
 * @since 1.4
 */
class ModuleUser_EntityUserPrivacy extends Entity
{
    /** @var int Доступ к данным — по умолчанию (видно всем) */
    public const USER_PRIVACY_ALL = 0;
    /** @var int Доступ к данным — только зарегистрированным */
    public const USER_PRIVACY_REGISTERED = 1;
    /** @var int Доступ к данным — только друзьям */
    public const USER_PRIVACY_FRIENDS = 2;
    /** @var int Доступ к данным — никому */
    public const USER_PRIVACY_NOBODY = 4;

    /** Возвращает ID пользователя
     * @return int
     */
    public function getUserId(): int
    {
        return $this->_getDataOne('user_id') ?? 0;
    }
    /** Приватность профиля.
     * 0 — по умолчанию (видно всем)
     * 1 — только зарегистрированным
     * 2 — только друзьям
     * 4 — никому
     * @return int
     */
    public function getProfileAccessLevel(): int
    {
        return $this->_getDataOne('privacy_profile') ?? 0;
    }
    /** Приватность данных о дате регистрации и последнего входа, а так же подписках в профиле.
     * 0 — по умолчанию (видно всем)
     * 1 — только зарегистрированным
     * 2 — только друзьям
     * 4 — никому
     * @return int
     */
    public function getProfileDataAccessLevel(): int
    {
        return $this->_getDataOne('privacy_profile_data') ?? 0;
    }
    /** Приватность раздела "Активность" профиля.
     * 0 — по умолчанию (видно всем)
     * 1 — только зарегистрированным
     * 2 — только друзьям
     * 4 — никому
     * @return int
     */
    public function getActivitiesAccessLevel(): int
    {
        return $this->_getDataOne('privacy_activities') ?? 0;
    }
    /** Приватность раздела "Друзья" профиля.
     * 0 — по умолчанию (видно всем)
     * 1 — только зарегистрированным
     * 2 — только друзьям
     * 4 — никому
     * @return int
     */
    public function getFriendlistAccessLevel(): int
    {
        return $this->_getDataOne('privacy_friendlist') ?? 0;
    }
    /** Приватность раздела "Избранное" профиля.
     * 0 — по умолчанию (видно всем)
     * 1 — только зарегистрированным
     * 2 — только друзьям
     * 4 — никому
     * @return int
     */
    public function getFavoritesAccessLevel(): int
    {
        return $this->_getDataOne('privacy_favorites') ?? 0;
    }


    /** Настройка приватности данных о датах регистрации, последнего захода и подписок в профиле
     * 0 — по умолчанию (видно всем)
     * 1 — только зарегистрированным
     * 2 — только друзьям
     * 4 — никому
     * @param int $data */
    public function setProfileAccessLevel(int $data)
    {
        $this->_aData['privacy_profile']=$data;
    }
    /** Настройка приватности профиля
     * 0 — по умолчанию (видно всем)
     * 1 — только зарегистрированным
     * 2 — только друзьям
     * 4 — никому
     * @param int $data */
    public function setProfileDataAccessLevel(int $data)
    {
        $this->_aData['privacy_profile_data']=$data;
    }
    /** Настройка приватности раздела "Активность" в профиле
     * 0 — по умолчанию (видно всем)
     * 1 — только зарегистрированным
     * 2 — только друзьям
     * 4 — никому
     * @param int $data */
    public function setActivitiesAccessLevel(int $data)
    {
        $this->_aData['privacy_activities']=$data;
    }
    /** Настройка приватности списка друзей в профиле
     * 0 — по умолчанию (видно всем)
     * 1 — только зарегистрированным
     * 2 — только друзьям
     * 4 — никому
     * @param int $data */
    public function setFriendlistAccessLevel(int $data)
    {
        $this->_aData['privacy_friendlist']=$data;
    }
    /** Настройка приватности раздела "Избранное"
     * 0 — по умолчанию (видно всем)
     * 1 — только зарегистрированным
     * 2 — только друзьям
     * 4 — никому
     * @param int $data */
    public function setFavoritesAccessLevel(int $data)
    {
        $this->_aData['privacy_favorites']=$data;
    }
}
