import React from 'react';
import ReactDOM from "react-dom";
import ImageUploader from "../components/ImageUploader";
import {gettext} from "../core/lang";
import io from "../lib/io";
import routes from "../lib/routes";
import {notice} from "../core/messages";
import {registry} from "../core/tools";

export const init = () => {

    const avatarNode = document.querySelector('#component-image-uploader--avatar');
    if(avatarNode)
        ReactDOM.render(
            <ImageUploader
                img={registry.get('component-image-uploader--avatar--url')}
                defaultImg={registry.get('component-image-uploader--avatar--url-default')}
                imgIsEmpty={!parseInt(registry.get('component-image-uploader--avatar--is-set'))}
                width={600}
                height={400}
                minImageWidth={100}
                minImageHeight={100}
                containerClass="avatar-change"
                previewClass="avatar"
                linkClass="link-dotted"
                border={50}
                aspectRatio={1}
                title={gettext('upload_avatar')}
                onUpload={uploadAvatar}
                onRemove={removeAvatar}
            />,
            avatarNode
        );

    const photoNode = document.querySelector('#component-image-uploader--photo');
    if(photoNode)
        ReactDOM.render(
            <ImageUploader
                img={registry.get('component-image-uploader--photo--url')}
                defaultImg={registry.get('component-image-uploader--photo--url-default')}
                imgIsEmpty={!parseInt(registry.get('component-image-uploader--photo--is-set'))}
                maxWidth={parseInt(registry.get('component-image-uploader--photo--max-width'))}
                width={480}
                height={480}
                minImageWidth={100}
                minImageHeight={100}
                containerClass="profile-photo-change"
                previewClass="profile-photo"
                linkClass="link-dotted"
                border={50}
                title={gettext('upload_photo')}
                onUpload={uploadPhoto}
                onRemove={removePhoto}
            />,
            photoNode
        );

    document.querySelector('#profile-settings-privacy').onsubmit = saveSettingsPrivacy;
};

const uploadAvatar = (base64URI) => {
    return io
        .post(routes.image.uploadAvatar, {image: base64URI})
        .then((result) => {
            notice(null, 'Аватар обновлён');
            return result;
        });
};

const removeAvatar = () => {
    return io
        .post(routes.image.removeAvatar)
        .then((result) => {
            notice(null, 'Аватар удалён');
            return result;
        });
};

const uploadPhoto = (base64URI) => {
    return io
        .post(routes.image.uploadFoto, {image: base64URI})
        .then((result) => {
            notice(null, 'Картинка профиля обновлена');
            return result;
        });
};

const removePhoto = () => {
    return io
        .post(routes.image.removeFoto)
        .then((result) => {
            notice(null, 'Картинка профиля удалена');
            return result;
        });
};

const saveSettingsPrivacy = function(e) { console.log(this);
    e.preventDefault();
    const ajax = new io.AjaxLoader();
    ajax.setForm(this).post().then((result) => { notice(result.sMsgTitle, result.sMsg);});
};
