import React from 'react';
import {error} from 'core/messages';
import {gettext} from 'core/lang';


export default class FileUpload extends React.PureComponent {
    inputField = null;

    static propTypes = {
        title: React.PropTypes.string,
        className: React.PropTypes.string,
        acceptMime: React.PropTypes.string,
        handleUpload: React.PropTypes.func,
        handleChange: React.PropTypes.func,
    };
    filetype = '';

    onLoad = (data) => {
        const image = new Image();

        const blob = new Blob([new Uint8Array(data.target.result)],{type:this.filetype});
        image.src = URL.createObjectURL(blob);//data.target.result;

        image.onload = () => {
            this.inputField.value = '';
            this.filetype = '';
            this.props.handleChange(blob/*image.src*//*data.target.result*/, image.width, image.height);
            URL.revokeObjectURL(image.src);
            document.body.style.cursor = '';
        }
    };

    setRef = (inputField) => {
        if (inputField) this.inputField = inputField;
    };

    handleUpload = ({target:{files}}) => {
        const file = files[0];
        if (!file) return;
        this.filetype = file.type || '';
        document.body.style.cursor = 'wait';
        const reader = new FileReader();
        reader.onload = this.onLoad;
        reader.onerror = (e) => {
            error(gettext('fileupload_error'), reader.error+' '+gettext('try_another_file'));
            document.body.style.cursor = '';
        };
        reader.readAsArrayBuffer(file);//DataURL(file);
    };

    render() {
        const name = String(Math.random() * 10**20);
        return <div>
            <input
                className="react-fileinput"
                onChange={this.handleUpload}
                ref={this.setRef}
                type="file"
                id={name}
                accept={this.props.acceptMime}
            />
            <label htmlFor={name} className={this.props.className}>{this.props.title}</label>
        </div>
    }
}
