import xhr from 'xhr';
import {merge} from 'lodash';
import {error, notice} from 'core/messages';

import {gettext} from 'core/lang';


function encodeParams(payload) {
    return Object
        .keys(payload)
        .map((k) => `${encodeURIComponent(k)}=${encodeURIComponent(payload[k])}`)
        .join('&');
}

function post(route, data = {}) {
    const options = {};
    if(data instanceof FormData) {
        data.set('security_ls_key',window.LIVESTREET_SECURITY_KEY);
        options['body'] = data;
    }
    else {
        options['body'] = encodeParams(merge(data, {security_ls_key: window.LIVESTREET_SECURITY_KEY}));
        options['headers'] = {'Content-Type': 'application/x-www-form-urlencoded'};
    }
    return new Promise((resolve, reject) => xhr.post(
        route, options,
        (err, res, body) => err ? reject(err) : resolve(res)
    ))
    .catch(() => error(gettext('network_error'), gettext('data_not_send')))
    .then((result) => {
        if (!result) {
            error(gettext('network_error'), gettext('try_later'));
            return Promise.reject(result);
        }
        if (result.body === 'Hacking attempt!') {
            error(null, result.body);
            return Promise.reject(result);
        }

        try {
            const jsonBody = JSON.parse(result.body);
            if (jsonBody.bStateError) {
                error(jsonBody.sMsgTitle, jsonBody.sMsg);
                return Promise.reject(result);
            }
            return Promise.resolve(jsonBody);
        } catch {
            // ignore json parse error
        }

        return Promise.resolve(result);
    })
}

function AjaxLoader() {
    let form, elementWithLoading = 'button[type="submit"]', url = undefined, formData = undefined, loading = false;

    /** @param {HTMLFormElement} value */
    this.setForm = (value) => { form = value; return this; }
    /** @param {HTMLElement} value */
    this.setElementWithLoading = (value) => { elementWithLoading = value; return this; }
    /** @param {string} value */
    this.setUrl = (value) => { url = value; return this; }
    /** @param {Object|FormData} value */
    this.setData = (value) => { formData = value; return this; }

    this.post = async () => {
        if(loading) return false;
        if(!form && formData === undefined) throw new Error('No HTML form or direct data is specified.');
        const ewl = form.querySelector(':scope '+elementWithLoading);
        if(form && ewl !== null) ewl.classList.add('loading');
        loading = true;
        if(formData === undefined)
            formData = new FormData(form);
        if(url === undefined)
            url = form.action || '';
        try {
            return await post(url,formData);
        }
        finally {
            loading = false;
            if(form && ewl !== null) ewl.classList.remove('loading');
        }
    }
}

export default {post, encodeParams, AjaxLoader};
