#!/usr/bin/env -S bash -m

# Аргументы для запуска (порядок важен): [dev|- [<custom_name>|- [rebuild|- [log|-]]]].
# Запуск без аргументов соберёт и запустит Табун в "боевом" режиме.
# dev соберёт и запустит Табун для разработки.
# С помощью <custom_name> можно задать другое имя проекта (и результирующих контейнеров). По умолчанию tabun.
# rebuild вызовет принудительную пересборку (с использованием кеширования, см. ниже как нивелировать) и не запустит контейнеры.
# log выведет подробный лог в файл build.log в папке проекта (в .gitignore отсутствует, поэтому удаляйте этот файл, чтобы он не попал в репозиторий).
# При сборке для разработки в проекте появится или пересоздастся папка vendor с библиотеками, подтянутыми композером, изменения в ней не влияют на сервер.

# Команды для ручной очистки окружения докера перед чистой пересборкой:
# docker builder prune -af (чистит кеш сборки, не влияет на собранные образы и контейнеры)
# docker container rm <проект-сервис-индекс> (удаляет отдельно взятый контейнер с названием проект-сервис-индекс)
# или docker container prune (удаляет все контейнеры, когда-либо созданные на хосте (не только Табунские!))
# docker volume rm <название> (удаляет отдельно взятое хранилище, не привязанное ни к одному контейнеру)
# или docker volume prune (удаляет все тома, не привязанные ни к одному контейнеру (не только Табунские!))
# Если производятся изменения в скриптах сборки, влияющие на стартовое содержимое томов, их нужно предварительно удалить.

function onSigInt() {
  # вывод от down будет тот же, что и от работающего в данный момент up
  docker compose -p $1 -f $2 down > /dev/null
  exit 0
}

composefile=docker-compose.yml
projectname='tabun'
if [[ "$2" != "-" ]] && [[ -n "$2" ]]; then
  projectname=$2
fi
if [[ $1 = "dev" ]]; then
  composefile=docker-compose.develop.yml
  projectname=${projectname}'-dev'
  if [[ $3 = "rebuild" ]]; then
    rm -rf ./vendor
  fi
fi
rebuild=""
if [[ $3 = "rebuild" ]]; then
  rebuild="--build --no-start"
else
  trap "onSigInt '${projectname}' '${composefile}'" SIGINT
fi
if [[ $4 = "log" ]]; then
  BUILDKIT_PROGRESS=plain docker compose -p $projectname -f $composefile up $rebuild > build.log 2>&1 &
else
  docker compose -p $projectname -f $composefile up $rebuild &
fi
# для пересборки запуск композа в фоне не нужен, но чтобы не писать docker compose 4 раза, проще всегда запускать в фоне и для пересборки сразу же возвращаться к нему
if [[ $3 = "rebuild" ]]; then
  fg %%
  if [[ $1 = "dev" ]]; then
    docker cp ${projectname}-app-1:/app/vendor/ ./vendor
    chown -R --reference=. ./vendor
  fi
else
  # в остальных случаях ждём когда завершится процесс докера (это произойдёт в обработчике, который выше повесили на SIGINT)
  wait
fi
